package com.example.config;

import com.example.entity.AccountDetails;
import com.example.entity.RestBean;
import com.example.entity.vo.response.AuthorizeVo;
import com.example.filter.JwtAuthenticationFilter;
import com.example.service.AccountService;
import com.example.utils.JwtUtils;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.config.annotation.web.reactive.EnableWebFluxSecurity;
import org.springframework.security.config.web.server.SecurityWebFiltersOrder;
import org.springframework.security.config.web.server.ServerHttpSecurity;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.server.SecurityWebFilterChain;
import org.springframework.security.web.server.WebFilterExchange;
import org.springframework.security.web.server.context.NoOpServerSecurityContextRepository;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;


@Slf4j
@Configuration
@EnableWebFluxSecurity  // 启用 WebFlux 安全配置
public class SecurityConfiguration {

    @Resource
    private JwtUtils utils;

    @Resource
    private AccountService service;

    @Resource
    private JwtAuthenticationFilter filter;

    @Bean
    public SecurityWebFilterChain filterChain(ServerHttpSecurity http) {
        http
                // 禁用 CSRF 防护，适用于无状态的 API
                .csrf(ServerHttpSecurity.CsrfSpec::disable)
                // 禁用 HTTP 基本认证
                .httpBasic(ServerHttpSecurity.HttpBasicSpec::disable)
                // 禁用表单登录
                .formLogin(ServerHttpSecurity.FormLoginSpec::disable)
                // 使用 NoOpServerSecurityContextRepository，意味着不存储安全上下文，适用于 stateless 应用（JWT）
                .securityContextRepository(NoOpServerSecurityContextRepository.getInstance())
                .authorizeExchange(exchanges -> exchanges
                        .pathMatchers("/chat/auth/**").permitAll()
                        .anyExchange().authenticated()
                )
                // 配置自定义登录处理
                .formLogin(form -> form
                        .loginPage("/chat/auth/login")
                        .authenticationSuccessHandler(this::onAuthenticationSuccess)
                        .authenticationFailureHandler(this::onAuthenticationFailure)
                )
                // 配置异常处理（认证失败、权限不足等）
                .exceptionHandling(conf -> conf
                        .authenticationEntryPoint(this::onUnauthorized)
                        .accessDeniedHandler(this::onAccessDenied)
                )
                // 配置登出处理
                .logout(logout -> logout
                        .logoutUrl("/chat/auth/logout")
                        .logoutSuccessHandler(this::onLogoutSuccess)
                )
                .addFilterAt(filter, SecurityWebFiltersOrder.AUTHENTICATION);
        return http.build();  // 返回构建的安全过滤链
    }

    /**
     * 登录成功处理
     * @param webFilterExchange WebFilterExchange，包含了请求和响应的上下文
     * @param authentication Authentication，表示当前用户的认证信息
     * @return Mono<Void>，用于异步响应
     */
    private Mono<Void> onAuthenticationSuccess(WebFilterExchange webFilterExchange,
                                               Authentication authentication) {
        ServerHttpResponse response = webFilterExchange.getExchange().getResponse();
        AccountDetails user = (AccountDetails) authentication.getPrincipal();

        return service.findAccountByNameOrEmail(user.getUsername())
                .flatMap(account -> {
                    String token = utils.generateJwt(user, account.id(), account.username());

                    AuthorizeVo vo = account.asViewObject(AuthorizeVo.class, v -> {
                        v.setExpireTime(utils.expireTime());
                        v.setToken(token);
                    });

                    return RestBean.writeSuccessToResponse(response, vo);
                })
                .onErrorResume(e -> {
                    log.error("认证处理失败", e);
                    return RestBean.writeFailureToResponse(response, 401,  e.getMessage());
                });
    }

    /**
     * 登出成功处理
     * @param exchange WebFilterExchange，包含了请求和响应的上下文
     * @param authentication Authentication，表示当前用户的认证信息
     * @return Mono<Void>，用于异步响应
     */
    public Mono<Void> onLogoutSuccess(WebFilterExchange exchange,
                                      Authentication authentication) {
        ServerHttpResponse response = exchange.getExchange().getResponse();

        // 如果有JWT令牌，将其加入黑名单
        String authHeader = exchange.getExchange().getRequest().getHeaders().getFirst("Authorization");
        if (authHeader != null) {
            utils.invalidateJwt(authHeader);
        }

        return RestBean.writeSuccessToResponse(response);
    }

    /**
     * 登录失败处理
     * @param webFilterExchange WebFilterExchange，包含了请求和响应的上下文
     * @param exception AuthenticationException，表示认证失败的异常
     * @return Mono<Void>，用于异步响应
     */
    private Mono<Void> onAuthenticationFailure(WebFilterExchange webFilterExchange,
                                               AuthenticationException exception) {
        ServerHttpResponse response = webFilterExchange.getExchange().getResponse();

        return RestBean.writeUnauthorizedToResponse(response, exception.getMessage());
    }

    /**
     * 未认证处理
     * @param exchange ServerWebExchange，包含了请求和响应的上下文
     * @param exception AuthenticationException，表示认证异常
     * @return Mono<Void>，用于异步响应
     */
    private Mono<Void> onUnauthorized(ServerWebExchange exchange,
                                      AuthenticationException exception) {
        ServerHttpResponse response = exchange.getResponse();

        return RestBean.writeUnauthorizedToResponse(response, exception.getMessage());
    }

    /**
     * 访问权限不足处理
     * @param exchange ServerWebExchange，包含了请求和响应的上下文
     * @param denied AccessDeniedException，表示访问被拒绝的异常
     * @return Mono<Void>，用于异步响应
     */
    private Mono<Void> onAccessDenied(ServerWebExchange exchange,
                                      AccessDeniedException denied) {
        ServerHttpResponse response = exchange.getResponse();

        return RestBean.writeFailureToResponse(response, 403, denied.getMessage());
    }
}
