package com.example.entity;

import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONWriter;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpResponse;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;

public record RestBean<T>(int code, T data, String message) {

    /**
     * 创建成功响应（带数据）
     */
    public static <T> RestBean<T> success(T data) {
        return new RestBean<>(HttpStatus.OK.value(), data, "请求成功");
    }

    /**
     * 创建成功响应（无数据）
     */
    public static <T> RestBean<T> success() {
        return success(null);
    }

    /**
     * 创建未授权响应
     */
    public static <T> RestBean<T> unauthorized(String message) {
        return failure(HttpStatus.UNAUTHORIZED.value(), message);
    }

    /**
     * 创建禁止访问响应
     */
    public static <T> RestBean<T> forbidden(String message) {
        return failure(HttpStatus.FORBIDDEN.value(), message);
    }

    /**
     * 创建自定义错误响应
     */
    public static <T> RestBean<T> failure(int code, String message) {
        return new RestBean<>(code, null, message);
    }

    /**
     * 转换为JSON字符串
     */
    public String asJsonString() {
        return JSONObject.toJSONString(this, JSONWriter.Feature.WriteNulls);
    }

    /**
     * 将该对象写入响应
     * 适用于WebFlux响应式环境
     */
    public Mono<Void> writeToResponse(ServerHttpResponse response) {
        response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
        response.setStatusCode(HttpStatus.valueOf(this.code));

        byte[] bytes = this.asJsonString().getBytes(StandardCharsets.UTF_8);
        DataBuffer buffer = response.bufferFactory().wrap(bytes);
        return response.writeWith(Mono.just(buffer));
    }

    /**
     * 将成功响应写入响应对象（带数据）
     * @param response 服务器HTTP响应对象
     * @param data 响应数据
     */
    public static <T> Mono<Void> writeSuccessToResponse(ServerHttpResponse response, T data) {
        return success(data).writeToResponse(response);
    }

    /**
     * 将成功响应写入响应对象（无数据）
     * @param response 服务器HTTP响应对象
     */
    public static Mono<Void> writeSuccessToResponse(ServerHttpResponse response) {
        return success().writeToResponse(response);
    }

    /**
     * 将失败响应写入响应对象
     * @param response 服务器HTTP响应对象
     * @param code HTTP状态码
     * @param message 错误消息
     */
    public static Mono<Void> writeFailureToResponse(ServerHttpResponse response, int code, String message) {
        return failure(code, message).writeToResponse(response);
    }

    /**
     * 将未授权响应写入响应对象
     * @param response 服务器HTTP响应对象
     * @param message 错误消息
     */
    public static Mono<Void> writeUnauthorizedToResponse(ServerHttpResponse response, String message) {
        return unauthorized(message).writeToResponse(response);
    }
}

