package com.example.filter;

import com.auth0.jwt.interfaces.DecodedJWT;
import com.example.utils.JwtUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.ReactiveSecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import org.springframework.web.server.WebFilter;
import org.springframework.web.server.WebFilterChain;
import reactor.core.publisher.Mono;

@Slf4j
@Component
@RequiredArgsConstructor
public class JwtAuthenticationFilter implements WebFilter {

    private final JwtUtils jwtUtils;

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, WebFilterChain chain) {
        // 获取Authorization头
        String authHeader = exchange.getRequest().getHeaders().getFirst("Authorization");

        // 如果没有Authorization头或者不是需要认证的路径，直接放行
        if (authHeader == null || shouldSkipAuthentication(exchange)) {
            return chain.filter(exchange);
        }

        // 解析JWT令牌
        DecodedJWT jwt = jwtUtils.resolveJwt(authHeader);
        if (jwt != null) {
            // 从JWT中提取用户信息
            UserDetails userDetails = jwtUtils.toUser(jwt);
            // 创建认证对象
            UsernamePasswordAuthenticationToken authentication =
                    new UsernamePasswordAuthenticationToken(
                            userDetails, null, userDetails.getAuthorities());

            // 将认证信息设置到SecurityContext中
            return chain.filter(exchange)
                    .contextWrite(ReactiveSecurityContextHolder.withAuthentication(authentication));
        }

        // 如果认证失败，继续过滤器链，后续会由Spring Security的认证失败处理器处理
        return chain.filter(exchange);
    }

    private boolean shouldSkipAuthentication(ServerWebExchange exchange) {
        String path = exchange.getRequest().getURI().getPath();
        // 不需要认证的路径
        return path.startsWith("/chat/auth/") ||
                path.equals("/chat/auth/login") ||
                path.equals("/chat/auth/register") ||
                exchange.getRequest().getMethod().name().equals("OPTIONS");
    }
}
