package com.example.utils;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTVerificationException;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Component;

import java.util.Calendar;
import java.util.Date;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Component
public class JwtUtils {
    // JWT的秘钥
    @Value("${spring.security.jwt.key}")
    private String key;

    // JWT的过期时间（天数）
    @Value("${spring.security.jwt.expire}")
    private int expire;

    @Resource
    private StringRedisTemplate template;

    /**
     * 生成JWT令牌
     *
     * @param details 用户详情
     * @param id 用户ID
     * @param username 用户名
     * @return 生成的JWT字符串
     */
    public String generateJwt(UserDetails details, int id,
                              String username) {
        Algorithm algorithm = Algorithm.HMAC256(key);
        return JWT.create()
                .withJWTId(UUID.randomUUID().toString())
                .withClaim("id", id)
                .withClaim("username", username)
                .withClaim("authorities", details.getAuthorities()
                        .stream().map(GrantedAuthority::getAuthority).toList()
                )
                .withIssuedAt(new Date())
                .sign(algorithm);
    }

    /**
     * 获取JWT的过期时间
     *
     * @return 过期时间
     */
    public Date expireTime() {
        Calendar calendar = Calendar.getInstance();
        // 计算过期时间（天数转换为小时）
        calendar.add(Calendar.HOUR, expire * 24);
        return calendar.getTime();
    }

    /**
     * 使JWT失效
     *
     * @param headerToken 请求头中的JWT令牌
     * @return 是否成功使JWT失效
     */
    public boolean invalidateJwt(String headerToken) {
        String token = this.convertToken(headerToken);
        if (token == null) return false;
        Algorithm algorithm = Algorithm.HMAC256(key);
        JWTVerifier jwtVerifier = JWT.require(algorithm).build();
        try {
            DecodedJWT jwt = jwtVerifier.verify(token);
            String id = jwt.getId();
            return this.deleteToken(id, jwt.getExpiresAt());
        } catch (JWTVerificationException e) {
            return false;
        }
    }

    /**
     * 将JWT的ID和过期时间存入Redis黑名单
     * @param uuid JWT的ID
     * @param expireTime JWT的过期时间
     * @return 是否删除了Token
     */
    private boolean deleteToken(String uuid, Date expireTime) {
        if (this.invalidateJwt(uuid))
            return false;
        long expire = this.calculateExpireTime(expireTime);
        this.addTokenToBlacklist(uuid, expire);
        return true;
    }

    /**
     * 计算JWT的剩余有效时间
     * @param expireTime JWT的过期时间
     * @return 剩余的有效时间（单位：毫秒）
     */
    private long calculateExpireTime(Date expireTime) {
        Date now = new Date();
        return Math.max(expireTime.getTime() - now.getTime(), 0);
    }

     /**
     * 将JWT的ID和过期时间存入Redis黑名单
     * @param uuid JWT的ID，用于标识黑名单中的条目
     * @param expire JWT的剩余有效时间，单位：毫秒
     * 该方法将JWT的ID与过期时间存入Redis黑名单，
     * 使得黑名单中的JWT在过期后自动失效。
     */
    private void addTokenToBlacklist(String uuid, long expire) {
        template.opsForValue().set(
                Const.JWT_BLACK_LIST + uuid,
                "", expire, TimeUnit.MILLISECONDS
        );
    }
    /**
     * 判断JWT是否在黑名单中
     * @param uuid JWT的ID
     * @return 是否在黑名单中
     */
    private boolean isInvalidJwt(String uuid) {
        return template.hasKey(Const.JWT_BLACK_LIST + uuid);
    }

    /**
     * 解析JWT并验证其有效性
     *
     * @param headerToken 请求头中的JWT令牌
     * @return 解析后的DecodedJWT对象，如果无效返回null
     */
    public DecodedJWT resolveJwt(String headerToken) {
        String token = this.convertToken(headerToken);
        if (token == null) return null;
        Algorithm algorithm = Algorithm.HMAC256(key);
        JWTVerifier jwtVerifier = JWT.require(algorithm).build();
        try {
            DecodedJWT jwt = jwtVerifier.verify(token);
            if (this.isInvalidJwt(jwt.getId()))
                return null;
            Date expiresAt = jwt.getExpiresAt();
            return new Date().after(expiresAt) ? null : jwt;
        } catch (JWTVerificationException exception) {
            return null;
        }
    }

    /**
     * 转换请求头中的JWT令牌
     * @param headerToken 请求头中的JWT令牌
     * @return JWT内容信息
     */
    private String convertToken(String headerToken) {
        // 校验是否为Bearer令牌
        if (headerToken == null || !headerToken.startsWith("Bearer "))
            return null;
        return headerToken.substring(7);
    }

    /**
     * 从JWT中提取用户信息
     *
     * @param jwt 解析后的JWT对象
     * @return Spring Security的UserDetails对象
     */
    public UserDetails toUser(DecodedJWT jwt) {
        // 获取JWT的声明部分
        Map<String, Claim> claims = jwt.getClaims();
        return User
                .withUsername(claims.get("username").asString())
                .password("******")
                .authorities(claims.get("authorities").asArray(String.class))
                .build();
    }

    /**
     * 从JWT中提取用户ID
     *
     * @param jwt 解析后的JWT对象
     * @return 用户ID
     */
    public Integer toId(DecodedJWT jwt) {
        Map<String, Claim> claims = jwt.getClaims();
        return claims.get("id").asInt();
    }
}